# Emilie Lebarbier (Paris) and Meili Baragatti (Montpellier), 27/06/2017

library(wavethresh)

# Creation of a dictionary with the Haar functions
# arguments J and R: we want the Haar functions with resolutions from 2^R to 2^J
Dico <- function(J,R){
  N=2^(J+1)
  HaarMat=GenW(N,filter.number=1,family="DaubExPhase")
  seq=seq(J,R,-1)
  Nb.bases.j=c(2^seq)
  sum=sum(Nb.bases.j)
  Fh=HaarMat[,2:(sum[length(sum)]+1)]
  return(Dico.Funct=Fh)
}

# Evaluation of the functions in Dic.Funct at positions posx
Eval.posx<- function(Dic.Funct,posx,N){
  position.Funct=seq(1/N,1,1/N)  
  M=dim(Dic.Funct)[2]
  FhNew=matrix(0,ncol=M,nrow=length(posx))
  for (i in 1:length(posx)){
    FhNew[i,]=Dic.Funct[which(position.Funct>=posx[i])[1],]
  }
  return(Fhx=FhNew)
}

# Remove from the dictionary the bases which are null at all evaluated positions
Bases.NonNulles<- function(F.bases){
  nbases=dim(F.bases)[2]
  PresentBases=c(1:nbases)
  sumcolonnes = apply(F.bases,2,sum)
  toremove = which(sumcolonnes == 0)

  if (length(toremove) !=0){
  F.bases.Present=F.bases[,-toremove]
  PresentBases = PresentBases[-toremove]
  } else {
    F.bases.Present=F.bases
    PresentBases = PresentBases
  }
  
  list(Fh.P=F.bases.Present, PresentBases=PresentBases)
}


######################
# Creation and evaluation of the dictionary: we obtain a matrix F, with one column for each function in the dictionary, and one line for each evaluated position
# t: vector with the times of evaluation (this will be the support of the functions from the dictionary). This support will be transformed in [0,1] which is the support of the Haar functions
# J and R used to determine the resoultion of the Haar functions (see funtion Dico)
# invperiod: the maximum inverse period for the Fourier functions (sin and cos)
######################

CreationBases <-function(t,J,R,invperiode){
  Fh      = c()  # Haar
  Ff      = c()  # Fourier (sin,cos)

  n=max(t)
  posx=t/n

  # Haar
  Dic.Funct=Dico(J,R)
  Dic.Funct=abs(Dic.Funct)
  Fh=Eval.posx(Dic.Funct,posx,2^(J+1))
  namesHaar <- c()
  for (i in 1:dim(Fh)[2]){
    namesHaar <- c(namesHaar,paste("Haar",floor(i/2^J*100)/100,sep="")) 
  }

  ## Sin, cos
  namessincos <- c()
  for (i in 1:invperiode){
   Ff = cbind(Ff,sin(2*pi*i*posx),cos(2*pi*i*posx))
  namessincos <- c(namessincos,paste("sin(2pi*",i,"posx)",sep=""),paste("cos(2pi",i,"posx)",sep=""))
  }

  ## x and x^2
  x=posx
  xx=posx*posx
  
  ## constant term
  cste <- rep(1,length(posx))
  
  
  ## Creation of matrice F
  F.Bases=cbind(cste,Fh,Ff,x,xx)
#  F.Bases=cbind(cste,Ff,x,xx)
  colnames(F.Bases) = c("cste",namesHaar,namessincos,"x","xx")
  
  
  ## Removing columns corresponding to functions null at all evaluated positions
  temp <- Bases.NonNulles(F.Bases)
  F.Bases <- temp[[1]]
  FunctionsNonNulles <- temp[[2]]
  
  
  list(F.Bases=F.Bases,FunctionsNonNulles=FunctionsNonNulles)
}
