# Meili Baragatti, 23/08/2017, Montpellier France

rm(list=ls())

source("BayesSegmentation_functions.R")
source("CreationBases.R")

#######################################################################################
#                             Simulation of 1 series                                #
#######################################################################################

#### only 1 series with 4 segments, with a functional part, 100 points. We specify here only one variance for the error term only.
M <- 1
n <- 100
K <- 4
mu.choix <- c(0,1,2,3,4,5)
standard_deviation <- c(0.1)
varianceError <- standard_deviation^2
pHaar <- c(10,50,60)
p1 <- 3
p2 <- 5
nbsimu <- 1
series <- simuSeries(M,n,K,mu.choix,varianceError,pHaar,p1,p2)  

### Obtaining the series
simu_0.1 <- (series[[6]][3] + series[[6]][5] + series[[6]][6])[,1]


#######################################################################################
##### algo Metropolis_Hastings to estimates the breakpoints of the segmentation part and the functions present in the functional part
#######################################################################################

# parameters for the Metropolis-Hastings algo
itertot <- 20000
burnin <- 5000
lec1 <- 50
lec2 <- 50
nbseginit <- 3 
nbfuncinit <- 3
nbtochangegamma <- 2 
nbtochanger <- 2 
Pi <- c(1,rep(0.01,99))   
Fmatrix <- CreationBases(c(1:100),7,7,10)[[1]]
eta <- c(1,rep(0.01,dim(Fmatrix)[2]-1)) 

### Run the Metropolis-Hastings algo. For each level of noise (0.1, 0.5, 1 and 1.5) we run the algo on each of the simulated series
simu_0.1_resMH <- segmentation_bias_MH(simu_0.1, itertot, burnin,lec1,lec2, Fmatrix, nbseginit,nbtochangegamma,nbfuncinit,nbtochanger,Pi,eta,printiter=FALSE) 


#######################################################################################
### Looking at results
#######################################################################################


# plots of the series with noise
plot(simu_0.1,type='l',xlab="temps",ylab="observation")

# Remind that we have an artificial breakpoint at position 1
# position of breakpoints have been randomly chosen. 
# A breakpoint with a jump of 1 only is more difficult to detect than a breakpoint with a jump of 3, 4 or 5.

# Remind that the constant function is always selected by convention (it is in position 1).
# The true functions composing the bias are the Haar functions at position 10, Haar function at position 50, Haar function at position 60, and sin(2pi*5posx (corresponding to index 11, 51, 61 and 110 in the Fmatrix).
# Formula for the disturbance: f(t) = 0.3 sim (2pi*t/20) + 1.5 Haar10 - 2 Haar50 + 3 Haar60
# The Haar10 is more difficutl to detect than Haar50 or 60, because of its lower associated coefficient
# Sometimes the algorithm does not detect a Haar function and instead detect a breakpoint at the position of this Haar function. A confusion betwwen haar functions and breakpoints is possible.

truebreakpoints <- series[[3]][-K]
print(truebreakpoints)
truebias <- c(11,51,61,110)

# plots of breakpoints and functions setected during the post-burn-in iterations
par(mfrow=c(1,2))
plot(simu_0.1_resMH[[1]]/(itertot-burnin),xlab="gamma components",ylab="Posterior probabilities", main="Breakpoints selection")
abline(h=0.5,col="red",lwd=2)
plot(simu_0.1_resMH[[2]]/(itertot-burnin),xlab="r components",ylab="Posterior probabilities", main="Functions selection")
abline(h=0.5,col="red",lwd=2)
par(mfrow=c(1,1))
# nbactugamma
simu_0.1_resMH[[3]] 
# nbactur
simu_0.1_resMH[[4]]

# To decide which breakpoints to select: threshold 1/2
threshold <-7500/15000
breakpoints <- c(which(simu_0.1_resMH[[1]]/(itertot-burnin) > threshold))-1
nbselections <- simu_0.1_resMH[[1]][breakpoints+1]
cbind(breakpoints,nbselections)
gammahat <- rep(0,n)
for (i in (breakpoints+1)){gammahat[i] <- 1}

# To decide which functions to select: threshold 1/2
threshold <- 7500/15000
basefunctions <- c(which(simu_0.1_resMH[[2]]/(itertot-burnin) > threshold))
nbselections2 <- simu_0.1_resMH[[2]][basefunctions]
cbind(basefunctions,nbselections2,colnames(Fmatrix)[basefunctions])
rhat <- rep(0,dim(Fmatrix)[2])
for (i in basefunctions[-1]){rhat[i] <- 1}

# Once the breakpoints and functions have beeen selected, betagamma, lambdar et sigma2 can be estimated, 
# and the expectation of the series can be estimated in two steps: estimation of segmentation part, of the functional part, then add this two parts
priorminsigma2 <- 0.001
priormaxsigma2 <- 5
estim <- estimation_moy_biais(simu_0.1, itertot, burnin, lec1, lec2, Fmatrix, gammahat,rhat,priorminsigma2,priormaxsigma2,printiter=FALSE)
muest <- rep(0,length(breakpoints))
muest[1] <- estim[[4]][1]
reconstructionmu <- rep(0,dim(Fmatrix)[1])
if (length(breakpoints)>1){
  for (i in 2:length(breakpoints)){muest[i] <- estim[[4]][i]+muest[i-1]}
  compt <- 1
  for (i in 2:length(c(breakpoints,dim(Fmatrix)[1]))){
    reconstructionmu[compt:(c(breakpoints,dim(Fmatrix)[1])[i])] <- muest[i-1]
    compt <- compt + (c(breakpoints,dim(Fmatrix)[1])[i]-c(breakpoints,dim(Fmatrix)[1])[i-1])
  }
}
reconstructionf <- rep(0,dim(Fmatrix)[1])
if (length(basefunctions)>1){
  for (i in 2:length(basefunctions)){
    reconstructionf <- reconstructionf + estim[[5]][i-1]*Fmatrix[,basefunctions[i]]
  }   
}

reconstructiontot <- reconstructionmu + reconstructionf

# plot of the series and its estimated expectation
plot(simu_0.1,type='l',xlab="time",ylab="",cex=1,cex.lab=1,cex.main=1.5,cex.axis=1,lwd=2)
points(reconstructiontot,type='l',lwd=2,col="sienna2",lty=2)
segments(truebreakpoints[1],-1,truebreakpoints[1],7,col="red",lwd=2,lty=1)
segments(truebreakpoints[2],-1,truebreakpoints[2],7,col="red",lwd=2,lty=1)
segments(truebreakpoints[3],-1,truebreakpoints[3],7,col="red",lwd=2,lty=1)
legend("bottomright",legend = c("True series","Reconstruction"),lty=c(1,2), lwd=c(2,2), col=c("black","sienna2"),cex=c(1))

# plots of the segmentation part and of the functional part, and their estimations
par(mfrow=c(1,2))
plot(series[[6]][,3],type='l',xlab="time",ylab="",cex=1,cex.lab=1,cex.main=1.5,cex.axis=1,lwd=2,ylim=c(0,5))
points(reconstructionmu,type='l',col="sienna2",lty=2,lwd=2)
legend("topleft", legend = c("True mean", "Reconstruction"), lty=c(1,2), lwd=c(2,2), col=c("black","sienna2"),cex=1)
plot(series[[6]][,5],type='l',xlab="time",ylab="",cex=1,cex.lab=1,cex.main=1.5,cex.axis=1.7,lwd=2,ylim=c(-2,5))
points(reconstructionf,type='l',col="sienna2",lwd=2,lty=2)
legend("topleft", legend = c("True functionnal part", "Reconstruction"), lty=c(1,2), lwd=c(2,2), col=c("black","sienna2"),cex=1)
par(mfrow=c(1,1))



